/*----------------------------------------------------------------------------
 *      Name:    DEMO.C
 *      Purpose: USB HID Demo
 *      Version: V1.10
 *----------------------------------------------------------------------------
 *      This file is part of the uVision/ARM development tools.
 *      This software may only be used under the terms of a valid, current,
 *      end user licence from KEIL for a compatible version of KEIL software
 *      development tools. Nothing else gives you the right to this software.
 *
 *      Copyright (c) 2005-2007 Keil Software.
 *---------------------------------------------------------------------------*/

#include <stm32f10x_lib.h>                           // stm32f10x definitions

#include "type.h"

#include "usbreg.h"
#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"

#include "STM32_Init.h"                              // stm32 initialisation
#include "../MIELIB/inc/Interfaces.h"

BYTE InReport[8];				// HID Input Report
BYTE OutReport;					// HID Out Report

/*----------------------------------------------------------------------------
  Get HID Input Report -> InReport
 *----------------------------------------------------------------------------*/
void GetInReport (void) 
{
	/* Get Device Status */
		
		outMapleFrame.Command		=	GetCondition;
		outMapleFrame.DestAP		=	0x20;
		outMapleFrame.OriginAP	=	0x00;
		outMapleFrame.DataSize	=	0x01;

		outMapleFrame.Data.dwLine[0]=CONTROLLER;
	
		MapleSend(&outMapleFrame);
	
		if(MapleReceive(&inMapleFrame) == RCV_OK)
		{

			//Extract Analog stick
				InReport[0]=inMapleFrame.Data.bLine[4+4];
				InReport[1]=inMapleFrame.Data.bLine[4+5];
			//Extract another analog stick
				InReport[2] = inMapleFrame.Data.bLine[4+6];
				InReport[3] = inMapleFrame.Data.bLine[4+7];
			//Extract triggers
				InReport[5]=inMapleFrame.Data.bLine[4+2];
				InReport[4]=inMapleFrame.Data.bLine[4+3];

			//Extract Da Ua Ra La and buttons
				InReport[7]=~inMapleFrame.Data.bLine[4+0];;
				InReport[6]=~inMapleFrame.Data.bLine[4+1];

				return;
		}

		outMapleFrame.Command  = DeviceRequest;	//Command
		outMapleFrame.DestAP   = AP_DEVICEA;		//Device on Port A (DEVICE Address = 0x20)
		outMapleFrame.OriginAP = AP_PORTA;			//Our Device host Address (HOST Address = 0x00)
		outMapleFrame.DataSize = 0x00;					//No Addition data
		/* Send formed maple packet */
		MapleSend(&outMapleFrame);

		/* Wait for packet received, if receive in timeout OK, - continue */
		MapleReceive(&inMapleFrame);
		
}


/*----------------------------------------------------------------------------
  Set HID Output Report <- OutReport
 *----------------------------------------------------------------------------*/
void SetOutReport (void) 
{
//  GPIOB->ODR = (OutReport << 8);	                 // All LED Bits are set
}


/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) {

  stm32_Init ();        // STM32 Initialization

	InitMaplePins();
	DisMIECOMP();					//Disable MIE Companion Logic

	InReport[0] = 0x80;
	InReport[1] = 0x80;
	InReport[2] = 0x80;
	InReport[3] = 0x80;

	InReport[4] = 0x00;
	InReport[5] = 0x00;
	InReport[6] = 0x00;
	InReport[7] = 0x00;

  USB_Init();           // USB Initialization
  USB_Connect(TRUE);    // USB Connect

  while (1) {           // Loop forever
    ;
  } // end while
} // end main
