/*----------------------------------------------------------------------------
 *      U S B  -  K e r n e l
 *----------------------------------------------------------------------------
 *      Name:    USBDESC.C
 *      Purpose: USB Descriptors
 *      Version: V1.10
 *----------------------------------------------------------------------------
 *      This file is part of the uVision/ARM development tools.
 *      This software may only be used under the terms of a valid, current,
 *      end user licence from KEIL for a compatible version of KEIL software
 *      development tools. Nothing else gives you the right to use it.
 *
 *      Copyright (c) 2005-2007 Keil Software.
 *---------------------------------------------------------------------------*/

#include "type.h"

#include "usb.h"
#include "msc.h"
#include "usbcfg.h"
#include "usbdesc.h"


/* USB Standard Device Descriptor */
const BYTE USB_DeviceDescriptor[] = {
  USB_DEVICE_DESC_SIZE,              /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0110), /* 1.10 */          /* bcdUSB */
  0x00,                              /* bDeviceClass */
  0x00,                              /* bDeviceSubClass */
  0x00,                              /* bDeviceProtocol */
  USB_MAX_PACKET0,                   /* bMaxPacketSize0 */
  WBVAL(0x0000),                     /* idVendor */
  WBVAL(0x0002),                     /* idProduct */
  WBVAL(0x0100), /* 1.00 */          /* bcdDevice */
  0x00,                              /* iManufacturer */
  0x00,                              /* iProduct */
  0x00,                              /* iSerialNumber */
  0x01                               /* bNumConfigurations */
};

/* USB Configuration Descriptor */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor */
const BYTE USB_ConfigDescriptor[] = {
/* Configuration 1 */
  USB_CONFIGUARTION_DESC_SIZE,       /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(                             /* wTotalLength */
    1*USB_CONFIGUARTION_DESC_SIZE +
    1*USB_INTERFACE_DESC_SIZE     +
    2*USB_ENDPOINT_DESC_SIZE
  ),
  0x01,                              /* bNumInterfaces */
  0x01,                              /* bConfigurationValue */
  0x00,                              /* iConfiguration */
  USB_CONFIG_BUS_POWERED /*|*/       /* bmAttributes */
/*USB_CONFIG_REMOTE_WAKEUP*/,
  USB_CONFIG_POWER_MA(100),          /* bMaxPower */
/* Interface 0, Alternate Setting 0, MSC Class */
  USB_INTERFACE_DESC_SIZE,           /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,     /* bDescriptorType */
  0x00,                              /* bInterfaceNumber */
  0x00,                              /* bAlternateSetting */
  0x02,                              /* bNumEndpoints */
  USB_DEVICE_CLASS_STORAGE,          /* bInterfaceClass */
  MSC_SUBCLASS_SCSI,                 /* bInterfaceSubClass */
  MSC_PROTOCOL_BULK_ONLY,            /* bInterfaceProtocol */
  0x64,                              /* iInterface */
/* Bulk In Endpoint */
  USB_ENDPOINT_DESC_SIZE,            /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType */
  USB_ENDPOINT_IN(1),                /* bEndpointAddress */
  USB_ENDPOINT_TYPE_BULK,            /* bmAttributes */
  WBVAL(0x0040),                     /* wMaxPacketSize */
  0,                                 /* bInterval */
/* Bulk Out Endpoint */
  USB_ENDPOINT_DESC_SIZE,            /* bLength */
  USB_ENDPOINT_DESCRIPTOR_TYPE,      /* bDescriptorType */
  USB_ENDPOINT_OUT(2),               /* bEndpointAddress */
  USB_ENDPOINT_TYPE_BULK,            /* bmAttributes */
  WBVAL(0x0040),                     /* wMaxPacketSize */
  0,                                 /* bInterval */
/* Terminator */
  0                                  /* bLength */
};

/* USB String Descriptor (optional) */
const BYTE USB_StringDescriptor[] = {
/* Index 0x00: LANGID Codes */
  0x04,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0409), /* US English */    /* wLANGID */
/* Index 0x04: Manufacturer */
  0x2A,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
					//0x02
  'r',0,	//0x04
  '.',0,	//0x06
  'a',0,	//0x08
  '.',0,	//0x0A
  'm',0,	//0x0C
  '.',0,	//0x0E
  ' ',0,	//0x10
  'f',0,	//0x12
  'a',0,	//0x14
  'c',0,	//0x16
  't',0,	//0x18
  'o',0,	//0x1A
  'r',0,	//0x1C
  'y',0,	//0x1E
  ' ',0,	//0x20
  'S',0,	//0x22
  'T',0,	//0x24
  'O',0,	//0x26
  'R',0,	//0x28
  'E',0,	//0x2A
/* Index 0x20: Product */
  0x30,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
					//0x02
  'r',0,	//0x04
  '.',0,	//0x06
  'a',0,	//0x08
  '.',0,	//0x0A
  'm',0,	//0x0C
  '.',0,	//0x0E
  ' ',0,	//0x10
  'f',0,	//0x12
  'a',0,	//0x14
  'c',0,	//0x16
  't',0,	//0x18
  'o',0,	//0x1A
  'r',0,	//0x1C
  'y',0,	//0x1E
  ' ',0,	//0x20
  'D',0,	//0x22
  'C',0,	//0x24
  'B',0,	//0x26
  'R',0,	//0x28
  'I',0,	//0x2A
  'D',0,	//0x2C
  'G',0,	//0x2E
  'E',0,	//0x30
/* Index 0x4A: Serial Number */
  0x1A,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
					//0x02
  'D',0,	//0x04
  'C',0,	//0x06
  'M',0,	//0x08
  'I',0,	//0x0A
  'E',0,	//0x0C
  ' ',0,	//0x0E
  'v',0,	//0x10
  '.',0,	//0x12
  '1',0,	//0x14
  '.',0,	//0x16
  '0',0,	//0x18
  '0',0,	//0x1A
/* Index 0x64: Interface 0, Alternate Setting 0 */
  0x14,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
					//0x02
  'D',0,	//0x04
  'C',0,	//0x06
  'M',0,	//0x08
  'I',0,	//0x0A
  'E',0,	//0x0C
  ' ',0,	//0x0E
  'M',0,	//0x10
  'E',0,	//0x12
  'M',0,	//0x14
};
