/*----------------------------------------------------------------------------
 * Name:    mielib.c
 * Purpose: MAPLE BUS usage for STM32
 * Version: V1.00
 *----------------------------------------------------------------------------
 * This file is part of the OnAMAPLE tools.
 *
 * Copyright (c) 2007-2013 r.a.m. Factory STORE. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <stm32f10x_lib.h>    // STM32F10x Library Definitions
#include <string.h>						// memset definition
#include "STM32_Init.h"       // STM32 Initialization

#include "../MIELIB/inc/maplebus.h"
#include "../MIELIB/inc/interfaces.h"

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int main (void) 
{
	/* STM32 setup */
  stm32_Init ();
	/* Pullup IO ports*/
	InitMaplePins();
	/* Disable MIE Small Logic Companion */
	DisMIECOMP();
	/* Disable Interrupts processing */
	DisableMapleEXTI();

	/* Get Device Status */
	outMapleFrame.Command  = DeviceRequest;	//Command
	outMapleFrame.DestAP   = AP_DEVICEA;		//Device on Port A (DEVICE Address = 0x20)
	outMapleFrame.OriginAP = AP_PORTA;			//Our Device host Address (HOST Address = 0x00)
	outMapleFrame.DataSize = 0x00;					//No Addition data
	/* Send formed maple packet */
	MapleSend(&outMapleFrame);

	/* Wait for packet received, if receive in timeout OK, - continue */
	if(MapleReceive(&inMapleFrame) == RCV_OK)
	{
		/* Get All Status Request, both Fixed Device 
		Status and Free Device Status are received.*/
		outMapleFrame.Command = AllStatusRequest;
		/* Clearing the input buffer for clarity, not necessary need */
		memset(&inMapleFrame,0x00,1024);			
		/* Data Transmit to */
		MapleSend(&outMapleFrame);
		/* Data Receive from */
		MapleReceive(&inMapleFrame);

		/* Is Expansion device on port 1 present? */
		if(inMapleFrame.OriginAP & EXP_DEV1)		
		{
			/* Expansion Device connected on HOST PORT A and in Device on Port 1 */
			outMapleFrame.DestAP = AP_PORTA | EXP_DEV1;
			outMapleFrame.Command = DeviceRequest;
			/* Clearing the input buffer... */
			memset(&inMapleFrame,0x00,1024);
			/* Data Exchange */
			MapleSend(&outMapleFrame);
			MapleReceive(&inMapleFrame);

			/* Change just command, address and ports setup by previous...*/
			outMapleFrame.Command = AllStatusRequest;
			/* Clearing the input buffer... */
			memset(&inMapleFrame,0x00,1024);
			/* Data Exchange */
			MapleSend(&outMapleFrame);
			MapleReceive(&inMapleFrame);

			/* If Expansion device in Device port 1 has STORRAGE function, send some command*/
			if(outMapleFrame.Data.dwLine[0] & STORRAGE)
			{
				/* Get Media Info */
				outMapleFrame.Command  = GetMediaInfo;
				outMapleFrame.DestAP	 = AP_PORTA | EXP_DEV1;
				outMapleFrame.OriginAP = AP_PORTA;
				outMapleFrame.DataSize = 0x02;
				/* And Eddition data */
				outMapleFrame.Data.dwLine[0] = 0x00000002;
				outMapleFrame.Data.dwLine[1] = 0x00000000;
				
				memset(&inMapleFrame,0x00,1024);
				MapleSend(&outMapleFrame);
				MapleReceive(&inMapleFrame);
			}
		}

		/* Is Expansion device on port 2 present? */
		if(inMapleFrame.OriginAP & EXP_DEV2)
		{
			outMapleFrame.Command = DeviceRequest;
			outMapleFrame.DestAP = AP_PORTA | EXP_DEV2;						//Expansion Device on Device Port 2
			outMapleFrame.OriginAP = AP_PORTA;
			outMapleFrame.DataSize = 0x00;
			
			memset(&inMapleFrame,0x00,1024);
			MapleSend(&outMapleFrame);
			MapleReceive(&inMapleFrame);

			outMapleFrame.Command = AllStatusRequest;
			memset(&inMapleFrame,0x00,1024);
			MapleSend(&outMapleFrame);
			MapleReceive(&inMapleFrame);
		}
	}

  while (1) {                                     // Loop forever
  } // end while
} // end main
