#include <string.h>
#include <stdbool.h>

#include "MapleBus.h"
#include "MapleDevices.h"
#include "device.h"

//Current device Peripheral Function
unsigned long m_dwPeripheralFunction = 0x00000000;
//Our device address
unsigned char m_bOurDeviceAddress = 0xFF;
//Number of Transmits again
unsigned char m_bNumbersOfTramsmits = 0x03;
//Last Sended command
unsigned char m_bPreviousCommand = 0x00;
//Last Sended Data Size
unsigned char m_bPreviousDataSize = 0x00;

unsigned char FindExpDevices(void)
{
	#warning "No implement for expansion devices search."
	
	return 0x00;
}

/*Make our device address*/
unsigned char MakeOurDeviceAddress(unsigned char DeviceAddress)
{
	unsigned char ExpDevices=0;
	
	/* Detect Expansion devices on Expansion slots and return device presentation */
	ExpDevices = FindExpDevices();
	
	return ExpDevices|DeviceAddress;
}

/*Reset Device From Host*/
void ResetDevice(void)
{
}

void ProcessDeviceFrame(LPMAPLEFRAME MPFrame)
{
/*
#define	DeviceStatus				0x05
#define	DeviceAllStatus			0x06
#define	DeviceReply					0x07
#define	DataTransfer				0x08
#define	FunctionTypeUnknown	0xFE//Controller
#define	CommandUnknown			0xFD//Controller
#define	FileError						0xFB
#define	LCDError						0xFA
#define	ARError							0xF9
#define	TransmitAgain				0xFC//Host, Controller
*/
	//We receive our device address
	if(m_bOurDeviceAddress == 0xFF)
			m_bOurDeviceAddress = MPFrame->DestAP;
	
	//Is our maple perefirial already have address?
	if(MPFrame->DestAP != m_bOurDeviceAddress || m_bOurDeviceAddress == 0xFF)
		return;

	MPFrame->DestAP = MPFrame->OriginAP;
	MPFrame->OriginAP = MakeOurDeviceAddress(m_bOurDeviceAddress);

	switch(MPFrame->Command)
	{
		case	DeviceRequest:		//0x01
			MPFrame->Command = DeviceStatus;
			MPFrame->DataSize = sizeof(DeviceInformation)>>2;
			memcpy(MPFrame->Data.bLine,&DeviceInformation,sizeof(DeviceInformation));
			MapleSend(MPFrame);
			break;
		case	AllStatusRequest:	//0x02
			MPFrame->Command = DeviceAllStatus;
			MPFrame->DataSize = (sizeof(DeviceInformation)+sizeof(DeviceRevisionInfo))>>2;
			memcpy(MPFrame->Data.bLine,&DeviceInformation,sizeof(DeviceInformation)+sizeof(DeviceRevisionInfo));
			MapleSend(MPFrame);
			break;
		case	DeviceReset:		//0x03
			MPFrame->Command = DeviceReply;
			MPFrame->DataSize = 0;
			MapleSend(MPFrame);
			ResetDevice();
		//Hire we must processing "Peripheral is initialized".
			break;
		case	DeviceKill:			//0x04
		//Reply for this command
			MPFrame->Command = DeviceReply;
			MPFrame->DataSize = 0;
			MapleSend(MPFrame);
		//Stop interrupt processing
		//And halt
			while(1);
		case	GetCondition:		//0x09
		//Hire we get device status
		//Hire we copy our device condition
			if(GetDeviceCondition(MPFrame->Data.bLine))
			{
				MPFrame->Command = DataTransfer;
				MPFrame->DataSize = sizeof(CONTROLLERCONDITION)>>2;
				MapleSend(MPFrame);
			}
			else // Function Type unknown
			{
				MPFrame->Command = FunctionTypeUnknown;
				MPFrame->DataSize = 0;
				MapleSend(MPFrame);
			}

			break;
		case	TransmitAgain:		//0xFC
			if(m_bNumbersOfTramsmits != 0)
			{
				MPFrame->Command = m_bPreviousCommand;
				MPFrame->DataSize = m_bPreviousDataSize;
				MapleSend(MPFrame);
				m_bNumbersOfTramsmits--;
			}
			return;
		case	GetMediaInfo:		//0x0A
		case	BlockRead:			//0x0B
		case	BlockWrite:			//0x0C
		case	SetCondition:		//0x0E
		case	FT4Control:			//0x0F
		case	ARControl:			//0x10
		case	GetLastError:		//0x0D
		default:
				MPFrame->Command = FunctionTypeUnknown;
				MPFrame->DataSize = 0;
				MapleSend(MPFrame);
			 break;
	}
	m_bNumbersOfTramsmits = 3;
	m_bPreviousCommand = MPFrame->Command;
	m_bPreviousDataSize = MPFrame->DataSize;
}

/*
+0004	Function type	00h	The function type specifies
+0005		00h	the controller.
+0006		00h	
+0007		01h	
+0008	Read format	FFh	According to the controller format,
+0009		FFh	the controller's data is stored. The
+000a		00h	blocks to be used have been
+000b		00h	declared already by the device ID.
+000c		80h	
+000d		80h	
+000e		80h	
+000f		80h	
*/

unsigned char GetDeviceCondition(void *DeviceCond)
{
	//be shure what host is needed data from Controller, not other device!
	//==0x00000100 - Vibration pack... and others
	LPDEVICEINFORMATION lpDeviceCond = DeviceCond;

	if(lpDeviceCond->FT0 & CONTROLLER)
			return GetControllerCondition((LPCONTROLLERCONDITION)DeviceCond);
	
	if(lpDeviceCond->FT0 & KEYBOARD)
			return GetKeyboardCondition((LPKEYBOARDCONDITION)DeviceCond);
	
	if(lpDeviceCond->FT0 & MOUSE)
			return GetMouseCondition((LPMOUSECONDITION)DeviceCond);
	
	if(lpDeviceCond->FT0 & GUN)
	{
		#if defined (EN_GUN)
			#warning "No GUN condition processing supported!"
		#endif
	}

	return 0;
}
