#if !defined (MAPLEBUS)
#define MAPLEBUS

#include <stdbool.h>

extern unsigned char DeviceAddress;

//Command Codes
//HOST
#define	DeviceRequest				0x01
#define	AllStatusRequest		0x02
#define	DeviceReset					0x03
#define	DeviceKill					0x04
#define	GetCondition				0x09
#define	GetMediaInfo				0x0A
#define	BlockRead						0x0B
#define	BlockWrite					0x0C
#define	GetLastError				0x0D
#define	SetCondition				0x0E
#define	FT4Control					0x0F
#define	ARControl						0x10
#define	TransmitAgain				0xFC
//Device
#define	DeviceStatus				0x05
#define	DeviceAllStatus			0x06
#define	DeviceReply					0x07
#define	DataTransfer				0x08
#define	FunctionTypeUnknown	0xFE
#define	CommandUnknown			0xFD
#define	FileError						0xFB
#define	LCDError						0xFA
#define	ARError							0xF9
#define	TransmitAgain				0xFC

/*Receive frame error codes*/
#define	RCV_OK							0x00	//Receive OK
#define	RCV_CRC_ERROR				0x01	//CRC Detection error while receive data
#define	RCV_OVF_ERROR				0x02	//Receive Buffer Overfflow
#define	RCV_TIME_OUT				0x03	//Timeout exceeded while receive data wait
#define	RCV_NO_RXD					0x04	//Packet receive signalled, but START Pattern not detected
#define	RCV_FERR						0x05	//Frame Error

/*Host PORTS*/
#define AP_PORT_MASK				0xC0		//Extraction HOST PORT ID Mask

#define AP_PORTA						0x00		//HOST PORT 'A'
#define AP_PORTB						0x40		//HOST PORT 'B'
#define AP_PORTC						0x80		//HOST PORT 'C'
#define AP_PORTD						0xC0		//HOST PORT 'D'

/*Devices On Host PORTS*/
#define AP_DEVICE						0x20

#define AP_DEVICEA					(AP_PORTA | AP_DEVICE)		//Device on HOST PORT 'A'
#define AP_DEVICEB					(AP_PORTB | AP_DEVICE)		//Device on HOST PORT 'B'
#define AP_DEVICEC					(AP_PORTC | AP_DEVICE)		//Device on HOST PORT 'C'
#define AP_DEVICED					(AP_PORTD | AP_DEVICE)		//Device on HOST PORT 'D'

/*Expansion device present detection bits*/
#define	EXP_DEV1						0x01	//Expansion Device in Device Port '1'
#define	EXP_DEV2						0x02	//Expansion Device in Device Port '2'
#define	EXP_DEV3						0x04	//Expansion Device in Device Port '3'
#define	EXP_DEV4						0x08	//Expansion Device in Device Port '4'
#define	EXP_DEV5						0x10	//Expansion Device in Device Port '5'

/*Device functions*/
#define CONTROLLER	0x01000000		//FT0 : Controller Function
#define STORRAGE		0x02000000		//FT1 : Storage Function
#define LCD					0x04000000		//FT2 : B/W LCD Function
#define TIMER				0x08000000		//FT3 : Timer Function
#define AUDIO_INPUT	0x10000000		//FT4 : Audio input device Function
#define AR_GUN			0x20000000		//FT5 : AR-Gun Function
#define KEYBOARD		0x40000000		//FT6 : Keyboard
#define GUN					0x80000000		//FT7 : Light-Gun Function
#define VIBRATION		0x00010000		//FT8 : Vibration Function
#define MOUSE				0x00020000		//FT9 : Pointing Function
#define EXMEDIA			0x00040000		//FT10 : Exchange Media Function
#define CAMERA			0x00080000		//FT11 : Camera Device Functio

//Receive next frame time out
#define RCV_TIME_OUT_CNT		0x1999

typedef struct tagMapleDataFrame
{
/*Define counters for SOP and EOP Detect*/
	unsigned char Command;
	unsigned char DestAP;
	unsigned char OriginAP;
	unsigned char DataSize;
	 union{
			unsigned char bLine[1020];
			unsigned long dwLine[255];
	 }Data;
	unsigned char RxD_CRC;		//CRC Valid only on input frames
}MAPLEFRAME, *LPMAPLEFRAME;

extern MAPLEFRAME 		inMapleFrame;		//Input Maple Frame for Receive
extern MAPLEFRAME 		outMapleFrame;		//Output Maple Frame for Transmit

//Current device Peripheral Function
extern unsigned long m_dwPeripheralFunction;
//Our device address
extern unsigned char m_bOurDeviceAddress;
//Number of Transmits again
extern unsigned char m_bNumbersOfTramsmits;
//Last Sended command
extern unsigned char m_bPreviousCommand;
//Last Sended Data Size
extern unsigned char m_bPreviousDataSize;

/*Reset Device*/
void ResetDevice(void);
/*Send by Mapple Protocol*/
void MapleSend(LPMAPLEFRAME MapleFrame);
/*Receive Frame by Mapple Protocol*/
unsigned char MapleReceive(LPMAPLEFRAME MapleFrame);
/*Extract from DeviceInformation struct Peripheral Function Type, defined by user*/
void GetPeripheralFunctionType(void);
/*Process frame then use MIELIB for device build*/
void ProcessDeviceFrame(LPMAPLEFRAME MPFrame);

/*Functions for host processing*/
unsigned char *OneBlockRead(unsigned short wBlockNumber);
unsigned char OneBlockWrite(unsigned short wBlockNumber, unsigned char *lpbBuffer);


#endif
