 ;*----------------------------------------------------------------------------
 ;* Name:    miercv.s
 ;* Purpose: assembly file for Small MAPLE BUS logic receiver
 ;* Version: V1.00
 ;*----------------------------------------------------------------------------
 ;* This file is part of the OnAMAPLE tools.
 ;*
 ;* Copyright (c) 2007-2013 r.a.m. Factory STORE. All rights reserved.
 ;*----------------------------------------------------------------------------

CRL				EQU     0x00
CRH				EQU     0x04
IDR				EQU     0x08
ODR				EQU     0x0C
BSRR			EQU     0x10
BRR				EQU     0x14

;8 Bit Data port for receive
DATA_PORT		EQU		0x40010C00	;GPIO Port B 
EXTI			EQU		0x40010400	;External Interrupt port base address
IMR				EQU		0x00
PR				EQU		0x14

				AREA    |.text|, CODE, READONLY

; This assembly listing work with interrupts processing on lines SDCKA SDCKB

EXTI0_IRQHandler PROC	;Negedge - Receive New Byte
                EXPORT  EXTI0_IRQHandler		[WEAK] ;11 Clocks
				IMPORT	inMaplePointer
				IMPORT	inMapleEndPointer
				IMPORT	m_bCRC
				IMPORT	m_bDWCounter
				IMPORT	m_bOVF
				IMPORT	m_dwTemp
				
				push	{R1}
				LDR		R1, = (DATA_PORT)
				NOP
				LDR		R1, [R1,#IDR]
				NOP
				
				push	{R2-R5}
				
				;Clear Interrupt pending, by set PR Bit
				
				LDR		R3, = (EXTI)
				LDR		R2,[R3,#PR]
				ORR		R2,#0x01
				STR		R2,[R3,#PR]
				
				;Add pointer handler
				LDR		R2, = (inMaplePointer)
				LDR		R3, [R2]
				
				;CRC Check
				LDR		R4, = (m_bCRC)
				LDRB	R5, [R4]
				;Calc Current CRC
				EOR		R5, R1
				STRB	R5, [R4]
				
				;Store DWORD Data
				LDR		R5, = (m_dwTemp)
				STRB	R1, [R5]
				LDR		R1, [R5]
				;Check One Double Word counted
				LDR		R5, = (m_bDWCounter)
				LDRB	R4, [R5]
				AND		R4, #0xFF
				ADD		R4,#1

				CMPS	R4,#4
				BNE		NoDWCounted

				;Check Overflow
				LDR		R4, = (inMapleEndPointer)
				LDR		R4, [R4]
				CMPS	R3,R4
				BEQ		OverflowWhileDataReceive

				EOR		R4,R4
				STR		R1, [R3]
				ADD		R3,#4
				STR		R3, [R2]
				B		DWCounted
NoDWCounted				
				LSL		R1,#8
				LDR		R5, = (m_dwTemp)
				STR		R1, [R5]
DWCounted				
				LDR		R5, = (m_bDWCounter)
				STRB	R4, [R5]
				
				pop		{R2-R5}
				pop		{R1}
				BX      LR
OverflowWhileDataReceive	
				;Set overflow Byte
				LDR		R2, = (m_bOVF)
				MOV		R1, #0x01
				STRB	R1, [R2]
				
				LDR		R1, = (EXTI)
				LDR		R2,[R1,#IMR]
				AND		R2,#0xFFFFFFFC
				ORR		R3,#0x00000003
				STR		R3,[R1,#IMR]
				
				pop		{R2-R5}
				pop		{R1}
				BX      LR
                ENDP

EXTI1_IRQHandler PROC	;Posedge - Start Of Receive

                EXPORT  EXTI1_IRQHandler		[WEAK] ;11 Clocks
				IMPORT	inMapleFrame

				push	{R0-R2}
				;Mask Receive Start, and unmask Receive Data
				LDR		R1, = (EXTI)
				LDR		R0,[R1,#IMR]
				AND		R0,#0xFFFFFFFC
				ORR		R0,#0x00000001
				STR		R0,[R1,#IMR]
				;Mask all pending, Include Data RCV Pending
				ORR		R0,#0x03
				STR		R0,[R1,#PR]
				;Clear CRC Byte
				LDR		R1, = (m_bCRC)
				EOR		R2, R2
				STRB	R2, [R1]
				;Clear Overflow Byte
				LDR		R1, = (m_bOVF)
				STRB	R2, [R1]
				;Clear DW Counter
				LDR		R1, = (m_bDWCounter)
				STRB	R2, [R1]
				;Setup Mapple pointer
				
;				LDR		R1, = (inMaplePointer)
;				LDR		R2, = (inMapleFrame)
;				STR		R2, [R1]
				;Check Overflow
				
				;And exit
				pop		{R0-R2}
				BX      LR
                ENDP
	END
