 ;*----------------------------------------------------------------------------
 ;* Name:    mietrm.s
 ;* Purpose: assembly file for Small MAPLE BUS logic transmiter
 ;* Version: V1.00
 ;*----------------------------------------------------------------------------
 ;* This file is part of the OnAMAPLE tools.
 ;*
 ;* Copyright (c) 2007-2013 r.a.m. Factory STORE. All rights reserved.
 ;*----------------------------------------------------------------------------

CRL				EQU     0x00
CRH				EQU     0x04
IDR				EQU     0x08
ODR				EQU     0x0C
BSRR			EQU     0x10
BRR				EQU     0x14

SDCK_PORT		EQU		0x40010C00	;GPIO Port B

SDCK_IO_PORT	EQU	CRH			;Port For Setup IO
SDCK_IO_MASK	EQU	0xFFFFFF00	;IO MASK
SDCK_IO_OUT		EQU	0x00000022	;Set port output
SDCK_IO_IN		EQU	0x00000088	;set port input

SDCKA_PIN		EQU		(0x01<<8)	;PB8
SDCKB_PIN		EQU		(0x01<<9)	;PB9

;Line SDCKA UP
	MACRO
		SDCKA_UP
			LDR R5,[R4,#ODR]
			ORR	R5 ,#SDCKA_PIN
			STR R5,[R4,#ODR]
    MEND
;Line SDCKA DOWN
	MACRO
		SDCKA_DOWN
			LDR R5,[R4,#ODR]
			AND	R5 ,#~SDCKA_PIN
			STR R5,[R4,#ODR]
	MEND

;Line SDCKB UP
	MACRO
		SDCKB_UP
			LDR R6,[R4,#ODR]
			ORR	R6 ,#SDCKB_PIN
			STR R6,[R4,#ODR]
	MEND

;Line SDCKB DOWN
	MACRO
		SDCKB_DOWN
			LDR R6,[R4,#ODR]
			AND	R6 ,#~SDCKB_PIN
			STR R6,[R4,#ODR]
	MEND

;B Line is Data, A - Clock
	MACRO
$label          mBDataAClock
;Fase 1 (B - Data, A - Clock)
$label.mFase1
				LSLS	R0,#1			;Shift for extract data bit
				BCS.W	$label.SDBP			;Plus On SDCKB
				NOP						;Wait 1 cycle
				NOP						;Wait 1 cycle
				SDCKB_DOWN				;Data on SDCKB Is 0
				B		$label.mClockFase1
$label.SDBP	;Jump if data on SDCKB Is 1
				SDCKB_UP
				B		$label.mClockFase1
$label.mClockFase1
				NOP
				NOP
				NOP
				NOP
				SDCKA_DOWN
				B		$label.mWT00
$label.mWT00			B		$label.mWT01
$label.mWT01			NOP				
				SDCKB_UP
				B		$label.mFase2
				;Wait for Data Latch 9 clocks
$label.mFase2				
	MEND

;A line is data, B - Clock, part of another macro
	MACRO
$label			mADataBClockShort
;Fase 2 (A - Data, B - Clock)
$label.Fase2
				LSLS	R0,#1			;Shift for extract data bit
				BCS.W	$label.SDAP			;Plus On SDCKB
				NOP						;Wait 1 cycle
				NOP						;Wait 1 cycle
				SDCKA_DOWN				;Data on SDCKB Is 0
				B		$label.ClockFase2
$label.SDAP	;Jump if data on SDCKB Is 1
				SDCKA_UP
				B		$label.ClockFase2
$label.ClockFase2
	MEND

;A line is data, B - Clock	
	MACRO
$label			mADataBClock
$label			mADataBClockShort
				NOP
				NOP
				NOP
				NOP
				SDCKB_DOWN
				B		$label.WT10
$label.WT10		B		$label.WT11
$label.WT11		NOP				
				SDCKA_UP
	MEND	
                AREA    |.text|, CODE, READONLY
;**************** Send ****************
MapleSend PROC	;CKB_RIZE
				EXPORT  MapleSend
				push {R0-R8,LR}
				;Connection for INHRxD
				;PA4 - INHRxD
				;PB8 - SDCKA
				;PB9 - SDCKB
				;SEND Data Registers Legend
				;R0 - TMP
				;R1 - Data Frame Pointer
				;R2 - 32 Bit Counter
				;R3 - DATA Packet Counter
				;R4 - GPIOB Address
				;R5 - SDCKA Bit
				;R6 - SDCKB Bit
				;R7 - CRC Buffer
				;R8 - Byte Counter
;Setup IO for output
				MOV		R1,R0		;R1<-R0 - Data Frame Pointer
				LDR 	R2,=(0x8888);Init DWORD counter to 4x8 = 32 bits (2 bits send)

				LDR R4,= SDCK_PORT					;Load GPIOB Address
				LDR R0,= (SDCKA_PIN + SDCKB_PIN)	;Set bits 
				LDR	R5,= (SDCKA_PIN)
				LDR	R6,= (SDCKB_PIN)
				STR R0,[R4,#BSRR]					;Pull Up Ports
				LDR R0,[R4,#SDCK_IO_PORT]			;Load previous port config state
				;3 - Output Push Pull 50 MHz
				;4 - Floating Input
				;8 - Pull Up/Pull Down 50 MHz
				AND R0,#SDCK_IO_MASK
				ORR R0,#SDCK_IO_OUT
;Generate SOP Pattern
				STR R0,[R4,#SDCK_IO_PORT]			;Set port to output
				;Proccess to transmit SOP pattern
				;SDCKA --\_________________/--
				;SDCKB ----\_/-\_/-\_/-\_/----
				;wait and SDCKA Down: START
				BL.W	fSomeWait
				SDCKA_DOWN
				MOV		R3,#0x04
SOPPatternLoop
				;Wait and SDCKB Down: +1 = 1
				BL.W	fSomeWait
				SDCKB_DOWN
				;Wait and SDCKB Up:
				BL.W	fSomeWait
				SDCKB_UP
				SUBS	R3,#1
				BNE		SOPPatternLoop
				;wait and SDCKA UP: END
				BL.W	fSomeWait
				SDCKA_UP
				;Process Send Data Packet.
				;Next transition (bit set to SDCKB) must be after 9 Clocks
;Send Data
				LDR		R3,[R1]		;Load Bytes Count into R3
				LSR		R3,#24
				AND 	R3,#0xFF	;Extract only size
				LDM		R1!,{R0}	; Load Data For Transmit into R0
									;and post increment R1
				MOV		R7,R0		;Init CRC
lRestart

OddBit			mBDataAClock
EvenBit			mADataBClockShort
;Byte almost sended we must recalc CRC and 
				LSRS	R2,#1			;>>1
				BEQ		LoadNextByte	;Check For New Word	
				bcc		noCRC
				EOR		R7,R0		;Calc CRC
				NOP
				SDCKB_DOWN
				B		ContinueTransmit
LoadNextByte
				NOP
				SDCKB_DOWN
				CBZ 	R3, TransmitionEnd
				SUB		R3,#0x01		;Flip Counter 
				LDR 	R2,=(0x8888)	;Init DWORD counter to 4x8 = 32 bits (2 bits send)
				LDM		R1!,{R0}	; Load Data For Transmit into R0
									;and post increment R1
				EOR		R7,R0			;Calc CRC
				NOP				
				SDCKA_UP

				B		lRestart
noCRC		
				SDCKB_DOWN
				B		WT10
ContinueTransmit
WT10			B		WT11
WT11			NOP				
				SDCKA_UP

				B		lRestart
TransmitionEnd
				B		WT20
WT20			MOV		R0,R7
				SDCKA_UP
;Send CRC
				MOV		R3,#04		;Load Count into R3
SendCRCLoop
CRCOddBit		mBDataAClock
CRCEvenBit		mADataBClock
				SUBS	R3,#01
				BNE		SendCRCLoop
;Generate EOP Pattern
				BL.W	fSomeWait
				SDCKB_UP
				BL.W	fSomeWait
				SDCKB_UP
				;Proccess to transmit SOP pattern
				;SDCKA ----\_/-\_/----
				;SDCKB --\_________/--
				;wait and SDCKB Down: START
				BL.W	fSomeWait
				SDCKB_DOWN
				MOV		R3,#0x02
EOPPatternLoop				
				;Wait and SDCKA Down: +1 = 1
				BL.W	fSomeWait
				SDCKA_DOWN
				;Wait and SDCKA Up:
				BL.W	fSomeWait
				SDCKA_UP
				SUBS	R3,#0x01
				BNE		EOPPatternLoop
				;wait and SDCKB UP: END
				BL.W	fSomeWait
				SDCKB_UP
;Setup IO for input
				LDR R0,[R4,#SDCK_IO_PORT]			;Load previous port config state
				AND R0,#SDCK_IO_MASK
				ORR R0,#SDCK_IO_IN
				STR R0,[R4,#SDCK_IO_PORT]			;Set port to output
				
				pop {R0-R8,LR}
				NOP
fSomeWait
				NOP
				BX   LR
				;End of transmit
                ENDP
		END
