/******************** (C) COPYRIGHT 2008 STMicroelectronics ********************
* File Name          : maplebus.c
* Author             : r.a.m. Factory STORE
* Version            : V1.0.0
* Date               : 06/13/2008
* Description        : This file includes the peripherals header files in the
*                      user application.
********************************************************************************
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

#include <stm32f10x_lib.h>                        // STM32F10x Library Definitions
#include "STM32_Reg.h"                            // missing bit definitions

#include "../inc/Interfaces.h"
#include "../inc/MapleBus.h"

/*Define Buffers For Mapple Transfer*/
MAPLEFRAME outMapleFrame;
MAPLEFRAME inMapleFrame;
/* Bytes counter for receiver */
volatile unsigned long m_dwTemp;
/* Pointer for receive buffer */
volatile LPMAPLEFRAME inMaplePointer;
volatile LPMAPLEFRAME inMapleEndPointer;

/*Calc CRC Byte, if it's not equal 0 after receive we have CRC Error*/
unsigned char m_bCRC;
/*Overflow Byte, if it's not equal 0 after receive we have Overflow Error*/
unsigned char m_bOVF;
/*Counter for byte receive*/
unsigned char m_bDWCounter;

#if !defined (MAPLE_HOST)
	#include "../inc/Devices.c"
#endif

#if defined (MAPLE_HOST)
	#include "../inc/Host.c"
	
/*Reset Pattern Send*/
void ResetDevice(void)
{
}
#endif

/*Maple receive algorithm*/
/*
0. Disable ALL receive interrupts
1. Attach input Buffer
2. Enable receive start detect interrupt
*/

unsigned char MapleReceive(LPMAPLEFRAME MapleFrame)
{
#if defined (MAPLE_HOST)
	unsigned long ulFrameTimeOut = 0;
#endif
	
	unsigned short wPortState = 0;
	unsigned char bRet = RCV_OK;

	inMaplePointer = MapleFrame;
	inMapleEndPointer = MapleFrame + 1024;
	
	DisableMapleEXTI();
	//Enable "receive start" detection interrupt
	EnRCVRDetect();
	//Enable "MIE Companion" logic
	EnMIECOMP();
	
#if !defined (MAPLE_HOST)
	//No Timeouts if we device
	while(!IsDTADetectEnabled());
#else
	//Host must have timeouts
	ulFrameTimeOut = RCV_TIME_OUT_CNT;

	while(!IsDTADetectEnabled() && ulFrameTimeOut)
	{																
			ulFrameTimeOut--;						
	}																
	
	if(!ulFrameTimeOut)	
	{
		//Receive ends with timeout
		DisableMapleEXTI();
		DisMIECOMP();
		inMaplePointer = MapleFrame;
		return RCV_TIME_OUT;
	}
#endif
	
	while(1)
	{
		wPortState = GPIOA->IDR;
		
		// Frame Error
		if(wPortState & FERR)
		{
			bRet = RCV_FERR;
			break;
		}
		//No Receive
		if(!(wPortState & RxD) && (wPortState & EOP))
		{
			bRet = RCV_NO_RXD;
			break;
		}
		
		//End of packet receive
		if(!(wPortState & EOP))
		{
			if(m_bCRC)
				bRet = RCV_CRC_ERROR;
			if(m_bOVF)
				bRet = RCV_OVF_ERROR;
			break;
		}
	}

	DisableMapleEXTI();
	DisMIECOMP();
	inMaplePointer = MapleFrame;
	return bRet;
}
